﻿//-----------------------------------------------------------------------
// <copyright file="Product.cs" company="deixei.com">
//     Copyright (c) deixei.com. All rights reserved.
// </copyright>
// <author>Marcio Parente</author>
//---------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Deixei.Domain
{
    public class Product : IDomainEntity, IArchivable, IAuditable
    {
        [Key]
        [Display(Name = "Id")]
        [DatabaseGeneratedAttribute(DatabaseGeneratedOption.Identity)]
        public int Id { get; set; }

        [Display(Name="Product Number")]
        [Required]
        public string Code { get; set; }
        public string Name { get; set; }
        public string Description { get; set; }


        public virtual ICollection<Supplier> Suppliers { get;set; }

        /// <summary>
        /// Archivable Region
        /// </summary>
        [Display(Name = "Deleted By", Description = "Username of the user that deleted this item.", ShortName = "Del.By")]
        public string DeletedBy { get; set; }

        [Display(Name = "Deleted Date", Description = "Date of when this item was deleted.", ShortName = "Del.Dt")]
        public DateTime? Deleted { get; set; }

        /// <summary>
        /// Auditable Region
        /// </summary>
        [Display(Name = "Created By", Description = "Username of the user that firts registered this item.", ShortName = "Crt.By")]
        public string CreatedBy { get; set; }

        [Display(Name = "Created Date", Description = "Date of when this item was firts registered.", ShortName = "Crt.Dt")]
        public DateTime Created { get; set; }

        [Display(Name = "Updated By", Description = "Username of the user that last changed this item.", ShortName = "Upd.By")]
        public string UpdatedBy { get; set; }

        [Display(Name = "Updated Date", Description = "Date of when this item was last changed.", ShortName = "Upd.Dt")]
        public DateTime? Updated { get; set; }

    }
}


// Possible missing
//public class ProductDefinition
//{  
//    public int ProductID { get; set; }
//    public string ProductNumber { get; set; }
//    public string Description { get; set; }
//    public virtual ICollection<ProductInstance> Instances { get; set; }
//}


//public class ProductInstance
//{
//    public int ProductInstanceID { get; set; }
//    public virtual Supplier Supplier { get; set; }
//    public virtual ProductDefinition Definition { get; set; }
//    public vritual ICollection<Ingredient> Ingredients { get; set; }
//}